/*---[ preferences.c ]------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The preferences notebook window
 *--------------------------------------------------------------------*/
#include <stdio.h>
#include <errno.h>

#include "globals.h"
#include "firestarter.h"
#include "preferences.h"

static GtkWidget *sections_clist;

GtkWidget *scriptentry;
GtkWidget *logentry;

GtkWidget *startup;
GtkWidget *stopatexit;
GtkWidget *resolve_names;

GtkWidget *redundant_filter;
GtkWidget *sep_filter;

//GtkWidget *showall;
GtkWidget *experiments;
GtkWidget *locatesbins;
GtkWidget *selectdeny;
GtkWidget *selectreject;

/* [ save_preferences ]
 * Save the preferences to the gnome registry
 */
static void
save_preferences (GnomePropertyBox *property_box, gint page_num, gint num)
{
/* TODO: This function is called 2 times when apply is pressed, why? */
/* General */
	gnome_config_set_bool ("/firestarter/Program/startup",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (startup)));

	gnome_config_set_bool ("/firestarter/Program/shutdown",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (stopatexit)));

	gnome_config_set_bool ("/firestarter/Program/resolve_names",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (resolve_names)));

/* Filters */

	gnome_config_set_bool ("/firestarter/Program/redundant_filter",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (redundant_filter)));

	gnome_config_set_bool ("/firestarter/Program/sep_filter",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sep_filter)));

/* Advanced */
/*	gnome_config_set_bool ("/firestarter/Druid/showall",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (showall)));
*/
	gnome_config_set_bool ("/firestarter/Druid/experiments",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (experiments)));

	gnome_config_set_bool ("/firestarter/Druid/locatesbins",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (locatesbins)));

	gnome_config_set_bool ("/firestarter/Druid/selectdeny",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (selectdeny)));

	gnome_config_set_bool ("/firestarter/Druid/selectreject",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (selectreject)));

	gnome_config_sync ();

	restart_firewall ();
}

/* [ create_general_page ]
 * Create the general property page
 */
static GtkWidget*
create_general_page (GtkWidget *pbox)
{
	GtkWidget *table;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox;

	table = gtk_table_new (1, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	frame = gtk_frame_new (_("General options"));
	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	startup = gtk_check_button_new_with_label (_(
		"Start firewall on program startup"));
	gtk_tooltips_set_tip (Firestarter.ttips, startup, _(
		"Check this and the firewall will be restarted each time the "
		"program starts. Enabled by default."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (startup),
		gnome_config_get_bool ("/firestarter/Program/startup=TRUE"));
	gtk_box_pack_start (GTK_BOX (vbox), startup, FALSE, FALSE, 0);

	stopatexit = gtk_check_button_new_with_label (_(
		"Stop firewall on program exit"));
	gtk_tooltips_set_tip (Firestarter.ttips, stopatexit, _(
		"Check this and the firewall will be stopped when you exit "
		"Firestarter. Enabled by default."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (stopatexit),
		gnome_config_get_bool ("/firestarter/Program/shutdown=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), stopatexit, FALSE, FALSE, 0);

	resolve_names = gtk_check_button_new_with_label (_("Always lookup hostname on hits"));
	gtk_tooltips_set_tip (Firestarter.ttips, resolve_names, _(
		"Check this and instead of the IP number you will see a valid hostname"
		" in the Hits view. This option uses some network bandwith."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resolve_names),
		gnome_config_get_bool ("/firestarter/Program/resolve_names=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), resolve_names, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	g_signal_connect_object (G_OBJECT (startup), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (stopatexit), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (resolve_names), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	return table;
}

/* [ create_filters_page ]
 * Create the general property page
 */
static GtkWidget*
create_filters_page (GtkWidget *pbox)
{
	GtkWidget *table;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox;

	table = gtk_table_new (1, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	frame = gtk_frame_new (_("Hit filtering options"));
	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	redundant_filter = gtk_check_button_new_with_label (_(
		"Filter out redundant hits"));
	gtk_tooltips_set_tip (Firestarter.ttips, redundant_filter, _(
		"Check this and the hits list will not show identical "
		"hits that succeed each other."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (redundant_filter),
		gnome_config_get_bool ("/firestarter/Program/redundant_filter=TRUE"));
	gtk_box_pack_start (GTK_BOX (vbox), redundant_filter, FALSE, FALSE, 0);

	sep_filter = gtk_check_button_new_with_label (_(
		"Filter out hits not meant for you"));
	gtk_tooltips_set_tip (Firestarter.ttips, sep_filter, _(
		"Check this and the hits list will not show hits "
		"that have a destination field other than your IP."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sep_filter),
		gnome_config_get_bool ("/firestarter/Program/sep_filter=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), sep_filter, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	g_signal_connect_object (G_OBJECT (redundant_filter), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (sep_filter), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	return (table);
}

static GtkWidget*
create_advanced_page (GtkWidget *pbox)
{
	GtkWidget *frame;
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *label;

	table = gtk_table_new (3, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	frame = gtk_frame_new (_("Wizard options"));
	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

/*	showall = gtk_check_button_new_with_label (_("Show every page in wizard"));
	gtk_tooltips_set_tip (Firestarter.ttips, showall, _(
		"Check this and every wizard page will be available in the wizard"), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showall),
		gnome_config_get_bool ("/firestarter/Druid/showall=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), showall, FALSE, FALSE, 0);
*/

	experiments = gtk_check_button_new_with_label (_("Enable Experimental Options"));
	gtk_tooltips_set_tip (Firestarter.ttips, experiments, _(
		"If you enable this, Firestarter can use some experimental patches for filtering."
		"Do not enable this option until you have read the manual for the pre-requisties"), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (experiments),
		gnome_config_get_bool ("/firestarter/Druid/experiments=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), experiments, FALSE, FALSE, 0);

	locatesbins = gtk_check_button_new_with_label (_("Try to detect the location of system binaries"));
	gtk_tooltips_set_tip (Firestarter.ttips, locatesbins, _(
		"Check this and Firestarter will search for the location of some "
		"needed binaries. Do not check unless you're having problems."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (locatesbins),
		gnome_config_get_bool ("/firestarter/Druid/locatesbins=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), locatesbins, FALSE, FALSE, 0);

	frame = gtk_frame_new (_("Preferred packet rejection method"));
	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	selectdeny = gtk_radio_button_new_with_label (NULL, _("Deny"));
	gtk_tooltips_set_tip (Firestarter.ttips, selectdeny, _(
		"Checking this box will simply ignore the unauthorized packets."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selectdeny),
		gnome_config_get_bool ("/firestarter/Druid/selectdeny=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), selectdeny, FALSE, FALSE, 0);

	selectreject = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (selectdeny),
		_("Reject"));
	gtk_tooltips_set_tip (Firestarter.ttips, selectreject, _(
		"Checking this box will reject the unauthorized packets"), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selectreject),
		gnome_config_get_bool ("/firestarter/Druid/selectreject=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), selectreject, FALSE, FALSE, 0);

	label = gtk_label_new (_("You must rerun the wizard for the changes to take effect"));
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);


/*	g_signal_connect_object (G_OBJECT (showall), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
*/
	g_signal_connect_object (G_OBJECT (experiments), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
 
	g_signal_connect_object (G_OBJECT (locatesbins), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	g_signal_connect_object (G_OBJECT (selectdeny), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	g_signal_connect_object (G_OBJECT (selectreject), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	return table;
}

static void
prefs_select_row_cb (GtkCList *clist,
                     gint row,
                     gint column,
                     GdkEventButton *event,
                     GtkWidget *pbox)
{
        gtk_notebook_set_page (GTK_NOTEBOOK (
		GNOME_PROPERTY_BOX (pbox)->notebook), row);
}

static int
prefs_add_section_page (gchar     *name,
                        GtkWidget *widget,
			GtkWidget *pbox)
{
	gchar *text[2] = {name, NULL};
	gtk_notebook_append_page (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pbox)->notebook),
				  widget, gtk_label_new (name));
	return gtk_clist_append (GTK_CLIST (sections_clist), text);
}

/* [ preferences_cb ]
 * Create the preferences property_box
 */
void
preferences_cb (GtkWidget *widget, gpointer data)
{
	static GtkWidget *pbox;
	GtkWidget *hbox;

	if (pbox != NULL) {
		g_assert (GTK_WIDGET_REALIZED (pbox));
		gdk_window_show (pbox->window);
		gdk_window_raise (pbox->window);

		return;
	}

	pbox = gnome_property_box_new ();
	gtk_window_set_title (GTK_WINDOW (pbox), _("Firestarter: Preferences"));

	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);

	sections_clist = gtk_clist_new (1);
	gtk_clist_set_selection_mode (GTK_CLIST (sections_clist),
				      GTK_SELECTION_BROWSE);

	g_signal_connect (G_OBJECT (sections_clist), "select_row",
			  G_CALLBACK (prefs_select_row_cb), pbox);

	prefs_add_section_page (_("General"), create_general_page (pbox), pbox);
	prefs_add_section_page (_("Filters"), create_filters_page (pbox), pbox);
	prefs_add_section_page (_("Advanced"), create_advanced_page (pbox), pbox);
	
	gtk_clist_columns_autosize (GTK_CLIST (sections_clist) );

	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pbox)->notebook), FALSE);
	gtk_notebook_set_show_border (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pbox)->notebook), FALSE);

	gtk_widget_ref (GNOME_PROPERTY_BOX (pbox)->notebook);

	gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (pbox)->vbox),
			      GNOME_PROPERTY_BOX (pbox)->notebook);

	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(pbox)->vbox),
			    hbox, TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX (hbox),
			    sections_clist,
			    TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX (hbox),
			    GNOME_PROPERTY_BOX (pbox)->notebook,
			    TRUE, TRUE, 0);

	gtk_widget_unref (GNOME_PROPERTY_BOX (pbox)->notebook);

	g_signal_connect (G_OBJECT (pbox), "apply",
	                  G_CALLBACK (save_preferences), NULL);

	g_signal_connect (G_OBJECT (pbox), "destroy",
			    G_CALLBACK (gtk_widget_destroyed), &pbox);

	gtk_widget_show_all (pbox);
}
