/*---[ scriptwriter.c ]-----------------------------------------------
 * Copyright (C) 2000-2002 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Creates the firewall script, based on wizard selections.
 *--------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>

#include "firestarter.h"
#include "globals.h"
#include "druid.h"
#include "util.h"
#include "druid-choices.h"
#include "scriptwriter.h"
#include "modify_ipup.h"
#include "netfilter-script.h"

/* [ check_file ]
 * Check that file exists, if not, create
 */
static void
check_file (const gchar *path)
{
	FILE *file = NULL;
	if ((fopen (path, "r") == NULL) && (errno == ENOENT)) {
	        if ((file = fopen (path, "w")) != NULL) {
			chmod (path, 00700);
			fclose (file);
        	}
	}
}

/* [ create_rules_files ]
 * Create the empty modrules scripts, unless already exists.
 */
static void
create_rules_files (void)
{
	check_file ((gchar *)get_file_path (RULETYPE_TRUSTED_HOST));
	check_file ((gchar *)get_file_path (RULETYPE_BLOCKED_HOST));
	check_file ((gchar *)get_file_path (RULETYPE_FORWARD));
	check_file ((gchar *)get_file_path (RULETYPE_OPEN_PORT));
	check_file ((gchar *)get_file_path (RULETYPE_STEALTHED_PORT));
	check_file ((gchar *)get_file_path (RULETYPE_BLOCKED_PORT));
}

/* [ write_script ]
 * Creates the shell script, called when the finish button is pressed
 */
void
write_script (Druid *data)
{
	write_netfilter_script (data);
	create_rules_files ();
	restart_firewall ();
	save_choices (data);

	if (gnome_config_get_bool ("/firestarter/Druid/startonppp=FALSE"))
		modify_ipup (TRUE);
	else
		modify_ipup (FALSE);

	if (gnome_config_get_bool ("/firestarter/Druid/firsttime=TRUE")) {
		/* Mark that the druid has been run at least once */
		gnome_config_set_bool ("/firestarter/Druid/firsttime", FALSE);
		gnome_config_sync ();

		/* Show the main interface */
		gtk_widget_show_all (Firestarter.window);
	}
}
